@@ -7,99 +7,102 @@ PICA::SRUSearchParser - Parse a SRU response in XML and extract PICA+ records.
 =cut
 
 use strict;
+use utf8;
 
-our $VERSION = "0.41";
+our $VERSION = "0.47";
+
+=head1 SYNOPSIS
+
+    $parser = PICA::SRUSearchParser->new();
+    $xmlparser = $parser->parse( $sru );
+
+    print "numberOfRecords: " . $parser->numberOfRecords . "\n";
+    print "resultSetId: " . $parser->resultSetId . "\n";
+    print "result: " . $xmlparser->counter() . "\n";
+
+=cut
 
 use Carp qw(croak);
 use PICA::XMLParser;
+use XML::Parser;
 
 =head1 METHODS
 
-=head2 new
+=head2 new ( [ $xmlparser ] )
 
-Creates a new Parser. See L<PICA::Parser> for a description of 
-parameters to define handlers (Field and Record).
+Creates a new XML parser to parse an SRU Search Response document.
+PICA Records are passed to a L<PICA::XMLParser> that must be provided.
 
 =cut
 
 sub new {
-    my ($class, %params) = @_;
+    my ($class, $xmlparser) = @_;
     $class = ref $class || $class;
 
+    $xmlparser = PICA::XMLParser->new()
+        unless UNIVERSAL::isa($xmlparser, "PICA::XMLParser");
+
     my $self = {
-        xmlparser => PICA::XMLParser->new( %params ),
-
-        char_data => "",              # not used yet
+        xmlparser => $xmlparser,
+        char_data => "",
         in_record => 0,
-
-        numberOfRecords => undef,     # not implemented yet
-        resultSetId => undef,         # this is needed for large result
-        nextRecordPosition => undef   # sets. not implemented yet.
+        numberOfRecords => undef,
+        currentNumber => 0,
+        resultSetId => undef,
     };
 
-    $self->{sruparser} = XML::Parser->new(
-       Handlers => {
-          Start => sub {$self->_start_handler(@_)},
-          End   => sub {$self->_end_handler(@_)},
-          Char  => sub {$self->_char_handler(@_)}
-          # TODO: Init and Final are never called. Do we need them?
+    return bless $self, $class;
+}
+
+=head2 parse( $document )
+
+Parse an SRU SearchRetrieve Response (given as XML document)
+and return the L<PICA::XMLParser> object that has been used.
+
+=cut
+
+sub parse {
+    my ($self, $document) = @_;
+    my $sruparser = XML::Parser->new(
+       Handlers => {    # memory leak
+          Start => sub {$self->StartTag(@_)},
+          End   => sub {$self->EndTag(@_)},
+          Char  => sub {$self->Text(@_)}
+#          # TODO: Init and Final are never called. Do we need them?
        }
     );
-
-    bless $self, $class;
-    return $self;
+    $self->{currentNumber} = 0;
+    $sruparser->parse($document);
+    return $self->{xmlparser};
 }
 
-=head2 parseResponse
-
-Parse an SRU SearchRetrieve Response given as an XML string.
-
-=cut
-
-sub parseResponse {
-    my ($self, $response) = @_;
-    $self->{sruparser}->parse($response);
-}
-
-=head2 counter
-
-Get the number of read records so far.
-
-=cut
-
-sub counter {
-   my $self = shift; 
-   return return $self->{xmlparser}->{read_counter};
-}
-
-=head2 empty
-
-Get the number of empty records that have been read so far.
-By default empty records are not passed to the record handler
-but counted.
-
-=cut
-
-sub empty {
-   my $self = shift; 
-   return $self->{xmlparser}->empty;
-}
-
-=head2 size
+=head2 numberOfRecords ()
 
 Get the total number of records in the SRU result set.
 The result set may be split into several chunks.
 
 =cut
 
-sub size {
+sub numberOfRecords {
     my $self = shift;
     return $self->{numberOfRecords};
 }
 
-=head2 resultSetId
+=head2 currentNumber ()
 
-Get the SRU resultSetId.
+Get the current number of records that has been passed.
+This is equal to or less then numberOfRecords.
+
+=cut
+
+sub currentNumber {
+    my $self = shift;
+    return $self->{currentNumber};
+}
+
+=head2 resultSetId ()
+
+Get the SRU resultSetId that has been parsed.
 
 =cut
 
@@ -110,9 +113,9 @@ sub resultSetId {
 
 =head1 PRIVATE HANDLERS
 
-Do not directly call this methods!
+This methods are private SAX handlers to parse the XML.
 
-=head2 _start_handler
+=head2 StartTag
 
 SAX handler for XML start tag. On PICA+ records this calls 
 the start handler of L<PICA::XMLParser>, outside of records
@@ -120,7 +123,7 @@ it parses the SRU response.
 
 =cut
 
-sub _start_handler {
+sub StartTag {
     my ($self, $parser, $name, %attrs) = @_;
     if ($self->{in_record}) {
         $self->{xmlparser}->start_handler($parser, $name, %attrs);
@@ -132,18 +135,19 @@ sub _start_handler {
     }
 }
 
-=head2 _end_handler
+=head2 EndTag
 
 SAX handler for XML end tag. On PICA+ records this calls 
 the end handler of L<PICA::XMLParser>.
 
 =cut
 
-sub _end_handler {
+sub EndTag {
     my ($self, $parser, $name) = @_;
 
     if ($self->{in_record}) {
         if ($name eq "srw:recordData") {
+            $self->{currentNumber}++;
             $self->{in_record} = 0;
         } else {
             $self->{xmlparser}->end_handler($parser, $name);
@@ -157,14 +161,14 @@ sub _end_handler {
     }
 }
 
-=head2 _char_handler
+=head2 Text
 
 SAX handler for XML character data. On PICA+ records this calls 
 the character data handler of L<PICA::XMLParser>.
 
 =cut
 
-sub _char_handler {
+sub Text {
     my ($self, $parser, $string) = @_;
 
     if ($self->{in_record}) {
@@ -176,29 +180,13 @@ sub _char_handler {
 
 1;
 
-=head1 TODO
-
-There seems to be a memory leak in the new() method, try
-while(1) { my $parser = PICA::SRUSearchParser->new(); }
-
-A method to get the parameters in the header (numberOfRecords, resultSetId...)
-is needed to get the number of records before actually parsing the result.
-
-Following requests of next records it not implemented yet.
-
-There is no check whether the SRU server supports pica format.
-
-Better error handling would be nice to skip invalid records but parse the rest.
-
-We need test cases.
-
 =head1 AUTHOR
 
 Jakob Voss C<< <jakob.voss@gbv.de> >>
 
 =head1 LICENSE
 
-Copyright (C) 2007-2009 by Verbundzentrale Gttingen (VZG) and Jakob Vo
+Copyright (C) 2007-2009 by Verbundzentrale Göttingen (VZG) and Jakob Voß
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself, either Perl version 5.8.8 or, at
